#include <gtk/gtk.h>
#include <stdio.h>

gboolean delete_event_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data )
{
    /* Received closure from the window manager */
    g_print("The window manager is asking to close this application\n");
    return( FALSE );  /* FALSE - do not prevent closure */
}


gboolean dialog_delete_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data )
{
   gtk_widget_hide( widget );
   return (TRUE); /* Prevent destruction */
}

void print_and_quit( GtkButton *was_clicked, gpointer user_data )
{
  /* Use glibs printf equivalent to print a message */
   g_print( "Thank you for using this program.\n");
   gtk_main_quit();
}

GtkWidget *append_menu_item( GtkMenu *menu, const gchar *text, 
                   GtkSignalFunc callback, gpointer user_data)
{
   GtkWidget *menu_item;
   if ( text )
   {
     menu_item = gtk_menu_item_new_with_label( text );
     gtk_object_set_data( GTK_OBJECT(menu), text, menu_item);
   }
   else
     menu_item = gtk_menu_item_new();

   gtk_menu_append( menu, menu_item );
   gtk_widget_show( menu_item );
   if (callback)
     gtk_signal_connect( GTK_OBJECT(menu_item), "activate", callback, user_data );
   return menu_item;
}

GtkWidget *create_file_and_menu( GtkMenuBar *menu_bar )
{
   GtkWidget *file_item, *file_menu, *exit_item;

    /* Create the File item and its menu widget */
     file_item = gtk_menu_item_new_with_label( "File" );
     gtk_object_set_data( GTK_OBJECT(menu_bar), "file_item", &file_item);
     gtk_menu_bar_append( GTK_MENU_BAR( menu_bar ), file_item );
       file_menu = gtk_menu_new();
       gtk_menu_item_set_submenu( GTK_MENU_ITEM( file_item ), file_menu );
    
       /* Create File menu items */
       append_menu_item( GTK_MENU(file_menu),  NULL, NULL, NULL );
       exit_item = append_menu_item( GTK_MENU(file_menu),  "Exit", 
                                     print_and_quit, NULL );
       gtk_object_set_data( GTK_OBJECT(file_item), "exit_item", exit_item );

   return file_item;
}

GtkWidget *create_top_window( const gchar*title )
{
  GtkWidget *top_widget;
  /* Create toplevel widget */
    top_widget = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_title( GTK_WINDOW( top_widget ), title );
    gtk_signal_connect( GTK_OBJECT( top_widget ), "delete-event",
                        GTK_SIGNAL_FUNC(delete_event_handler), NULL );
    gtk_signal_connect( GTK_OBJECT( top_widget ), "destroy",
                        GTK_SIGNAL_FUNC(print_and_quit), NULL );
    return top_widget;
}

