/*
 * Examples from Chapter 13.  
 * Calendar examples.
 */

/*
 * Use of the time() function requires the inclusion of the time.h header
 * file.
 */
#include <time.h>

#include <glib.h>

int main(
	int argc,
	char *argv[]
)
{
	GDate		*my_date;
	gchar		buf[100];


	/*
	 * Save the program name.
	 */
	g_set_prgname(argv[0]);


	/*
	 * Establish a new GDate structure.  Initialize it to the current date.
	 */
	my_date = g_date_new();
	g_date_set_time(my_date, time(NULL));
	g_date_strftime(buf,100,"Today is a %A, %x\n", my_date);
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, buf);


	/*
	 * Is this the first or last of the month?
	 */
	if ( g_date_is_first_of_month(my_date) )
		g_date_strftime(buf,100,
			"Today, %x, is the first of the month\n", my_date);
	else
		g_date_strftime(buf,100,
			"Today, %x, is not the first of the month\n", my_date);
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, buf);
	
	if ( g_date_is_last_of_month(my_date) )
		g_date_strftime(buf,100,
			"Today, %x, is the last of the month\n", my_date);
	else
		g_date_strftime(buf,100,
			"Today, %x, is not the last of the month\n", my_date);
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, buf);
	


	/*
	 * Jump ahead to 10 weeks.  What day is that?
	 */
	g_date_add_days(my_date, 10*7 );
	g_date_strftime(buf,100,"10 weeks from now would be %A, %x\n", my_date);
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, buf);
	g_date_free(my_date);


	/*
	 * Jump back 2 years, 3 months, 3 weeks, 5 days.  What day is that?
	 */
	my_date = g_date_new();
	g_date_set_time(my_date, time(NULL));
	g_date_subtract_years(my_date, 2 );
	g_date_subtract_months(my_date, 3 );
	g_date_subtract_days(my_date, 3*7 );
	g_date_subtract_days(my_date, 5 );
	g_date_strftime(buf,100,
		"Back 2 years, 3 months, 3 weeks and 5 days "
		"from now would be %A, %x\n", my_date);
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, buf);


	/*
	 * Is that year a leap year?
	 */
	if ( g_date_is_leap_year( g_date_year(my_date) ) )
		g_date_strftime(buf, 100, "%Y is a leap year", my_date);
	else
		g_date_strftime(buf, 100, "%Y is not a leap year", my_date);
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, buf);
      return 0;
}
