/*
 * Examples from Chapter 13.  
 * Logging examples which would cause this program to abort have been
 * commented out so that the whole program can be run.  Uncomment these
 * lines if you wish to see the output from logs which abort the program.
 */

#include <glib.h>

/*
 * Set a constant for use as my local domain for message logging.
 */
#define	MY_DOMAIN 	"GlibTests"


void
std_logs()
{
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, "Standard log(G_LOG_LEVEL_WARNING)");
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE, "Standard log(G_LOG_LEVEL_MESSAGE)");
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "Standard log(G_LOG_LEVEL_INFO)");
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Standard log(G_LOG_LEVEL_DEBUG)");
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_CRITICAL, "Standard log(G_LOG_LEVEL_CRITICAL)");

	/*
	 * This next one will cause a core dump because the G_LOG_LEVEL_ERROR
	 * level is set to be a fatal message by default.  We comment it out
	 * here so the program will run completely through.
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Standard log(G_LOG_LEVEL_ERROR)");
	 */
}

void
my_domain_logs()
{
	g_log(MY_DOMAIN, G_LOG_LEVEL_WARNING, "Standard log(G_LOG_LEVEL_WARNING)");
	g_log(MY_DOMAIN, G_LOG_LEVEL_MESSAGE, "Standard log(G_LOG_LEVEL_MESSAGE)");
	g_log(MY_DOMAIN, G_LOG_LEVEL_INFO, "Standard log(G_LOG_LEVEL_INFO)");
	g_log(MY_DOMAIN, G_LOG_LEVEL_DEBUG, "Standard log(G_LOG_LEVEL_DEBUG)");
	g_log(MY_DOMAIN, G_LOG_LEVEL_CRITICAL, "Standard log(G_LOG_LEVEL_CRITICAL)");

	/*
	 * This next one will cause a core dump because the G_LOG_LEVEL_ERROR
	 * level is set to be a fatal message by default.  We comment it out
	 * here so the program will run completely through.
	g_log(MY_DOMAIN, G_LOG_LEVEL_ERROR, "Standard log(G_LOG_LEVEL_ERROR)");
	 */
}

void
MyHandler(
	gchar *domain,
	GLogLevelFlags log_level,
	gchar *message,
	gpointer user_data
)
{
	gchar		*level_name;

	switch (log_level)
	{
		case G_LOG_LEVEL_WARNING:  level_name = "WARNING"; break;
		case G_LOG_LEVEL_MESSAGE:  level_name = "MESSAGE"; break;
		case G_LOG_LEVEL_INFO:     level_name = "INFO"; break;
		case G_LOG_LEVEL_DEBUG:    level_name = "DEBUG"; break;
		default:                   level_name = "OTHER"; break;
	}
	g_print("My Handler called on level %s.  Message = %s", 
		level_name, message);
}

void
convenience_functions()
{
	g_message("Generic Message");
	g_warning("Generic Warning");
	/*
	g_error("Generic Error");
	*/
}


int main()
{
	/*
	 * Log messages using the default handler.
	 */
	std_logs();

	/*
	 * Log messages using the default handler but with a local domain.
	 */
	my_domain_logs();

	/*
	 * Log messages using the default handler but with a local domain.
	 */
	convenience_functions();

	/*
	 * Install my own handler and try the logs with a local domain again.
	 */
	g_log_set_handler(
			MY_DOMAIN,
			G_LOG_LEVEL_MESSAGE, 
			(GLogFunc)MyHandler,
			GINT_TO_POINTER(0)
		);
	my_domain_logs();
        return 0;
}
