/*
 * Examples from Chapter 13.  
 * String handling examples.
 */

#include <glib.h>

#define LONG_STRING	\
"This is a long string <BREAK>\
which contains a number of line delimiters. <BREAK>\
It will be used to demonstrate <BREAK>\
line splitting <BREAK>\
and line joining. \
"

void
str_mods()
{
	gchar		*str;

	/*
	 * Simple g_snprintf() example.
	 */
	str = g_malloc(256);
	g_snprintf( str, 256, 
		"g_snprintf(): This is the program name: %s\n", g_get_prgname());
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, str);
	g_free(str);

	/*
	 * Simple g_snprintf() example.
	 */
	str = 
		g_strdup_printf( "g_strdup_printf(): This is the program name: %s\n", 
			g_get_prgname());
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, str);
	g_free(str);

	/*
	 * Chug and chomp.
	 */
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "\n\nCHUG and CHOMP\n");
	str = g_malloc(256);
	g_snprintf( str, 256, "\n\f\t This string has white space! \n\f\t");
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, str);
	g_strstrip( str );
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, str);
	g_free(str);

	/*
	 * String delimiter changes.
	 */
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "\n\nSTRING DELIMITERS\n");
	str = g_malloc(256);
	g_snprintf( str, 256, "Let's change a set of delimiters.");
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, str);
	g_strdelimit( str, "set", '*' );
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, str);
	g_free(str);

}


void
str_gens()
{
	gchar		*str;
	gchar		*new_str;
	gchar		**string_array;

	/*
	 * Splitting and joining strings.  Here we will break the original
	 * string into individual lines using the <BREAK> delimiter in the
	 * original text and replacing it with newlines (\n).
	 */
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "\n\nSTRING SPLITS and JOINS\n");
	str = g_malloc(256);
	g_snprintf( str, 256, LONG_STRING );
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "original string:\n%s\n", str);
	string_array = g_strsplit( str, "<BREAK>", 3 );
	new_str = g_strjoinv("\n", string_array);
	g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "new string:\n%s\n", new_str);
	g_free(new_str);
	g_strfreev(string_array);

}


int main(
	int argc,
	char *argv[]
)
{
	/*
	 * Save the program name.
	 */
	g_set_prgname(argv[0]);

	/*
	 * String Modification functions.
	 */
	str_mods();

	/*
	 * String Generation functions.
	 */
	str_gens();

     return 0;
}
