/*
 * Examples from Chapter 14.  
 * Hashes
 */

#include <stdlib.h>
#include <time.h>
#include <glib.h>

void
my_hash_foreach(
    gpointer    key,
    gpointer    value,
    gpointer    data
)
{
    gchar    buf[128];
    static gint    n = 1;

    g_snprintf(buf, 128, "%d: key: %s -- value: %s", 
            n++, (gchar *)key, (gchar *) value);
    g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, (gchar *)buf);
}

void
print_hash(
    GHashTable    *hash,
    gchar         *header
)
{
    gchar    *string;

    g_print("\n");
    string = g_strdup_printf("*** %s ***", header);
    g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, string);
    g_free( string );

    /*
     * Now print out the tree in order.
     */
    g_hash_table_foreach( hash, my_hash_foreach, (gpointer)0 );
}

gint
compare_hash(
    gconstpointer    key1,
    gconstpointer    key2
)
{
    if ( strcmp(key1, key2) )
        return TRUE;

    return FALSE;
}


gint
get_hash_val(
    gconstpointer    key
)
{
    gchar         buf[128];

    /*
     * We could write our own function to return a value based on the key,
     * but we're just going to call one of the default routines instead.
     * You could just have easily specified this default routine directly in 
     * the call to g_hash_table_new().
     */
    g_snprintf( buf, 128, 
        "Key:  %s  --  Hash Value: %d", (gchar *)key, g_str_hash( key ) );
    g_log( G_LOG_DOMAIN, G_LOG_LEVEL_INFO, buf );
    return ( g_str_hash( key ) );
}


int main()
{
    GHashTable    *hash = NULL;
    gchar         *string, *string2;
    gint          i;



    /*
     * Initialize random number generator.
     */
    srand((int)(
            (1 + (int)(500.0*rand()/(RAND_MAX+1.0))) * time(NULL)*time(NULL)));
    
    /*
     * Allocate a hash.
     */
    hash = g_hash_table_new((GHashFunc)g_str_hash, (GCompareFunc)g_str_equal);

    /*
     * Build a hash with 30 nodes, each with a random key.
     */
    for ( i=0; i<30; i++ )
    {
        string  = g_strdup_printf("%d", i);
        string2 = g_strdup_printf("The value associated with %d", i);
        g_hash_table_insert( hash, (gpointer)string, (gpointer)string2 );
    }
    
    /*
     * Now print out the list in order.
     */
    print_hash( hash, "Original Hash Table" );

    /*
     * Print the number of hashes.
     */
    string = g_strdup_printf("Size of hash table: %d", g_hash_table_size(hash));
    g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, string);
    g_free( string );
    
    /*
     * Look up a key.
     */
    string = g_strdup_printf("16");
    string2 = (gchar *)g_hash_table_lookup( hash, (gpointer)string );
    g_free( string );
    string = g_strdup_printf("key = 16, value = %s", string2);
    g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, string);
    g_free( string );
    
    /*
     * Free up the tree 
     */

   return 0;
}
