/*
 * Examples from Chapter 14.  
 * Linked lists.
 */

#include <glib.h>
#include <string.h>

void
print_slist(
    GSList    *slist,
    gchar        *header
)
{
    gchar        *string;
    GSList    *slist_ptr;
    int        i;

    g_print("\n");
    string = g_strdup_printf("*** %s ***", header);
    g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, string);
    g_free( string );

    /*
     * Now print out the list in order.
     */
    slist_ptr = slist;
    i = 1;
    while ( slist_ptr != NULL )
    {
        string = g_strdup_printf("Link %d: %s", i, (gchar *)slist_ptr->data);
        g_log(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, string);
        g_free( string );
        i++;
        slist_ptr = slist_ptr->next;
    }
}

gint
sort_slist(
    gconstpointer    data1,
    gconstpointer    data2
)
{
    gchar *str1 = (gchar *)data1;
    gchar *str2 = (gchar *)data2;

    return ( strcmp( str1, str2 ) );
}


void
process_slist(
    gpointer    data,
    gpointer    user_data
)
{
    g_strreverse( (gchar *)data );
    strncpy( (gchar *)data, (gchar *)user_data, 4 );
    g_strreverse( (gchar *)data );
}


int main()
{
    GSList    *slist = NULL;
    GSList    *slist2 = NULL;
    gchar        *string;
    int        i;

    /*
     * Initialize a singly linked list with 4 list elements, each with
     * pointers to text data.
     */
    for ( i=1; i<5; i++ )
    {
        string = g_strdup_printf("Text string %d", i);
        slist = g_slist_prepend( slist, string );
    }
    
    /*
     * Now print out the list in order.
     */
    print_slist( slist, "Original list" );

    /*
     * Insert generic items, one at index 2 and one at the end of the list.
     */
    string = g_strdup_printf("Generic string %d", ++i);
    slist = g_slist_insert( slist, string, 2 );
    print_slist( slist, "Inserted items, index 2" );
    string = g_strdup_printf("Generic string %d", ++i);
    slist = g_slist_insert( slist, string, -1 );
    print_slist( slist, "Inserted items, index -1" );

    /*
     * Reverse the ordering.
     */
    slist = g_slist_reverse( slist );
    print_slist( slist, "Reversed order" );

    /*
     * Sorted a copy of the list.
     */
    slist2 = g_slist_copy( slist );
    slist2 = g_slist_sort( slist2, sort_slist );
    print_slist( slist2, "Sorted order, copy of original" );

    /*
     * Process each link item in copy.
     */
    g_slist_foreach( slist2, process_slist, "ABCD" );
    print_slist( slist2, "Processed each element of copy" );

   return 0;
}
