#include <stdio.h>
#include <gtk/gtk.h>

gint counter = 0;

void button_remove_clicked(GtkButton *button, GtkWidget *list)
{
    GList *list_items = NULL;
    GtkWidget *list_item = NULL;
    
    /* Get the list item that corresponds to the clicked remove
       button. */
    list_item =
        gtk_object_get_data(GTK_OBJECT(button), "list_item");
    
    /* Create the GList of list items that will be removed, in
       this case it will only contain one. */
    list_items = g_list_prepend(list_items, list_item);
    
    /* Remove the list item. */
    gtk_list_remove_items(GTK_LIST(list), list_items);
}

GtkWidget *create_row(GtkWidget *list)
{
    GtkStyle *newstyle = NULL;
    GtkWidget *list_item = NULL;
    GtkWidget *hbox = NULL;
    GtkWidget *button = NULL;
    GtkWidget *label = NULL;
    gchar buffer[255];
    
    /* Create a new GtkListItem */
    list_item = gtk_list_item_new();
    
    /* Create the contents of the list item. */
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(list_item), hbox);
    
    button = gtk_button_new_with_label(" Remove ");
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
        (GtkSignalFunc)button_remove_clicked, list);
    gtk_object_set_data(GTK_OBJECT(button), "list_item",
        list_item);
    
    /* Make the selected button background color the same as
       the normal button background color. */
    gtk_widget_ensure_style(button);
    newstyle = gtk_style_copy(button->style);
    newstyle->bg[GTK_STATE_SELECTED] =
        newstyle->bg[GTK_STATE_NORMAL];
    gtk_widget_set_style(button, newstyle);
    
    /* Make the selected button label foreground color the same
       as the normal button label foreground color. */
    gtk_widget_ensure_style(GTK_BIN(button)->child);
    newstyle = gtk_style_copy(button->style);
    newstyle->fg[GTK_STATE_SELECTED] =
        newstyle->fg[GTK_STATE_NORMAL];
    gtk_widget_set_style(GTK_BIN(button)->child, newstyle);
    
    sprintf(buffer, "Label %d", counter);
    counter++;
    label = gtk_label_new(buffer);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
    
    gtk_object_set_data(GTK_OBJECT(list_item), "label", label);
    
    gtk_widget_show_all(list_item);
    
    return list_item;
}

GList* contruct_list_items(GtkWidget *list)
{
    GList *list_items = NULL;
    
    /* Construct 100 list items and put them in a GList. */
    while(counter < 100)
    {
        list_items = g_list_prepend(list_items,
            create_row(list));
    }
    
    list_items = g_list_reverse(list_items);
    
    return list_items;
}

void list_select_child(GtkList *list, GtkWidget *item,
         gpointer user_data)
{
    GtkWidget *label = NULL;
    gchar *label_text = NULL;
    GtkWidget *statusbar = NULL;
    gchar buffer[255];
    
    label = gtk_object_get_data(GTK_OBJECT(item), "label");
    gtk_label_get(GTK_LABEL(label), &label_text);
    
    statusbar = gtk_object_get_data(GTK_OBJECT(list),
        "statusbar");
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar), 1);
    sprintf(buffer, " \"%s\" selected", label_text);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, buffer);
}

void list_unselect_child(GtkList *list, GtkWidget *item,
         gpointer user_data)
{
    GtkWidget *label = NULL;
    gchar *label_text = NULL;
    GtkWidget *statusbar = NULL;
    gchar buffer[255];
    
    label = gtk_object_get_data(GTK_OBJECT(item), "label");
    gtk_label_get(GTK_LABEL(label), &label_text);
    
    statusbar = gtk_object_get_data(GTK_OBJECT(list),
        "statusbar");
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar), 1);
    sprintf(buffer, " \"%s\" unselected", label_text);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), 1, buffer);
}

void button_remove_selection_clicked(GtkButton *button,
         GtkWidget *list)
{
    GList *selected_items = NULL;
    GList *items_to_remove = NULL;
    
    /* Get the selected list items. */
    selected_items = GTK_LIST(list)->selection;
    
    /* Copy the obtained GList to another one since the
       selection GList is changed when removing items. */
    while(selected_items)
    {
        items_to_remove = g_list_prepend(items_to_remove,
            selected_items->data);
        selected_items = g_list_next(selected_items);
    }
    
    if(items_to_remove)
    {
        items_to_remove = g_list_reverse(items_to_remove);
    
        /* If there were items in the selection,remove them. */
        gtk_list_remove_items(GTK_LIST(list), items_to_remove);
    }
}

void button_add_clicked(GtkButton *button, GtkWidget *list)
{
    GtkWidget *focus_widget = NULL;
    gint focus = 0;
    GList *list_items = NULL;
    
    /* Get the currently focused list item. */
    focus_widget = GTK_LIST(list)->last_focus_child;
    if(focus_widget)
    {
        /* If there is a current focus, get the numeric
           position in the list. */
        focus = gtk_list_child_position(GTK_LIST(list),
            focus_widget);
    }
    
    /* Create the GList of items that will be inserted in the
       list. In this case, it's just one. */
    list_items = g_list_prepend(list_items, create_row(list));
    
    /* Insert the new row at the focus position or at the
       top. */
    gtk_list_insert_items(GTK_LIST(list), list_items, focus);
}

void button_select_all_clicked(GtkButton *button,
         GtkWidget *list)
{
    gtk_list_select_all(GTK_LIST(list));
}

void button_select_none_clicked(GtkButton *button,
         GtkWidget *list)
{
    gtk_list_unselect_all(GTK_LIST(list));
}

int main(int argc, char *argv[])
{
    GtkWidget *window = NULL;
    GtkWidget *vbox = NULL;
    GtkWidget *scrolled_window = NULL;
    GtkWidget *list = NULL;
    GtkWidget *table = NULL;
    GtkWidget *button_remove_selection = NULL;
    GtkWidget *button_add = NULL;
    GtkWidget *button_select_all = NULL;
    GtkWidget *button_select_none = NULL;
    GtkWidget *statusbar = NULL;
    GList *list_items = NULL;
    
    gtk_init(&argc, &argv);
    
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window),
        "Chapter 15 : example 1");
    gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE,
        FALSE);
    gtk_window_set_default_size(GTK_WINDOW(window), -1, 300);
    gtk_container_set_border_width(GTK_CONTAINER(window), 5);
    gtk_signal_connect(GTK_OBJECT(window), "delete_event",
        (GtkSignalFunc)gtk_exit, NULL);
    
    vbox = gtk_vbox_new(FALSE, 5);
    gtk_container_add(GTK_CONTAINER(window), vbox);
        
    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(scrolled_window),
         GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(vbox), scrolled_window,
        TRUE, TRUE, 0);
        
    list = gtk_list_new();
    gtk_list_set_selection_mode(GTK_LIST(list),
        GTK_SELECTION_MULTIPLE);
    /* Get the GList of initial list items. */
    list_items = contruct_list_items(list);
    /* Put those in the list. */
    gtk_list_append_items(GTK_LIST(list), list_items);
    /* Add the list the scrolled window. */
    gtk_scrolled_window_add_with_viewport(
        GTK_SCROLLED_WINDOW(scrolled_window), list);
    /* Attach the GtkList item focus to the adjustments of the
       GtkScrolledWindow. */
    gtk_container_set_focus_hadjustment(GTK_CONTAINER(list),
        gtk_scrolled_window_get_hadjustment
        (GTK_SCROLLED_WINDOW(scrolled_window)));
    gtk_container_set_focus_vadjustment(GTK_CONTAINER(list),
        gtk_scrolled_window_get_vadjustment
        (GTK_SCROLLED_WINDOW(scrolled_window)));
    /* Connect a signal handler to the select_child and
       unselect_child signals. */
    gtk_signal_connect(GTK_OBJECT(list), "select_child",
        (GtkSignalFunc)list_select_child, NULL);
    gtk_signal_connect(GTK_OBJECT(list), "unselect_child",
        (GtkSignalFunc)list_unselect_child, NULL);
    
    table = gtk_table_new(2, 2, TRUE);
    gtk_box_pack_start(GTK_BOX(vbox), table,
        FALSE, TRUE, 0);

    button_remove_selection =
        gtk_button_new_with_label(" Remove Selection ");
    gtk_signal_connect(GTK_OBJECT(button_remove_selection),
        "clicked",
        (GtkSignalFunc)button_remove_selection_clicked, list);
    gtk_table_attach(GTK_TABLE(table), button_remove_selection,
        0, 1, 0, 1,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    
    button_add = gtk_button_new_with_label(" Add Row ");
    gtk_signal_connect(GTK_OBJECT(button_add), "clicked",
        (GtkSignalFunc)button_add_clicked, list);
    gtk_table_attach(GTK_TABLE(table), button_add,
        1, 2, 0, 1,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    
    button_select_all =
        gtk_button_new_with_label(" Select All ");
    gtk_signal_connect(GTK_OBJECT(button_select_all),
        "clicked",
        (GtkSignalFunc)button_select_all_clicked, list);
    gtk_table_attach(GTK_TABLE(table), button_select_all,
        0, 1, 1, 2,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    
    button_select_none =
        gtk_button_new_with_label(" Select None ");
    gtk_signal_connect(GTK_OBJECT(button_select_none),
        "clicked",
        (GtkSignalFunc)button_select_none_clicked, list);
    gtk_table_attach(GTK_TABLE(table), button_select_none,
        1, 2, 1, 2,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL,
        GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    
    statusbar = gtk_statusbar_new();
    gtk_box_pack_start(GTK_BOX(vbox), statusbar,
        FALSE, TRUE, 0);
    gtk_object_set_data(GTK_OBJECT(list), "statusbar",
        statusbar);
    
    gtk_widget_show_all(window);
    
    gtk_main();
    
    return 0;
}
