#include <gtk/gtk.h>

gboolean OnWindowDeleteEvent(GtkWidget* widget, GdkEventAny* event, gpointer data)
{
	/*return false so that window will be destroyed*/
	return FALSE;
}

void OnWindowDestroy(GtkObject* object, gpointer data)
{
	/*exit main loop*/
	gtk_main_quit();
}

void OnButtonClicked(GtkButton* button, gpointer data)
{
	/*use a static variable to track button position*/
	static gboolean moved = FALSE;
	
	GtkLayout* layout;
	
	/*cast data to a GtkLayout*/
	layout = GTK_LAYOUT(data);
	
	/*move the button
		Note: it probably isn't nescessary to freeze the layout widget here,
	 	but better safe than sorry
	*/
	gtk_layout_freeze(layout);
	if(moved == FALSE)
	{
		gtk_layout_move(layout, GTK_WIDGET(button), 500, 500);
		moved = TRUE;
	}
	else
	{
		gtk_layout_move(layout, GTK_WIDGET(button), 0, 0);
		moved = FALSE;	
	}
	gtk_layout_thaw(layout);
}


int main (int argc, char** argv)
{
	GtkWidget* layout;
	GtkWidget* window;
	GtkWidget* table;
	GtkWidget* button;
	GtkWidget* vScrollbar;
	GtkWidget* hScrollbar;
	
	GtkAdjustment* vAdjust;
	GtkAdjustment* hAdjust;
	
	/*initialize gtk*/
	gtk_init(&argc, &argv);
	
	/*create the top level window*/
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "Layout Example");
	gtk_window_set_default_size(GTK_WINDOW(window), 300, 300);
	gtk_signal_connect(GTK_OBJECT(window), "delete-event", GTK_SIGNAL_FUNC(OnWindowDeleteEvent), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(OnWindowDestroy), NULL);
	
	/*create the table and pack into the window*/
	table = gtk_table_new(2, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(window), table);
	
	/*create the layout widget and pack into the table*/
	layout = gtk_layout_new(NULL, NULL);
	gtk_layout_set_size(GTK_LAYOUT(layout), 600, 600);
	gtk_table_attach(GTK_TABLE(table), layout, 0, 1, 0, 1, GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
	
	/*create the scrollbars and pack into the table*/
	vScrollbar = gtk_vscrollbar_new(NULL);
	gtk_table_attach(GTK_TABLE(table), vScrollbar, 1, 2, 0, 1, GTK_FILL|GTK_SHRINK, GTK_FILL|GTK_SHRINK, 0, 0);
	hScrollbar = gtk_hscrollbar_new(NULL);
	gtk_table_attach(GTK_TABLE(table), hScrollbar, 0, 1, 1, 2, GTK_FILL|GTK_SHRINK, GTK_FILL|GTK_SHRINK, 0, 0);	

	/*tell the scrollbars to use the layout widget's adjustments*/
	vAdjust = gtk_layout_get_vadjustment(GTK_LAYOUT(layout));
	gtk_range_set_adjustment(GTK_RANGE(vScrollbar), vAdjust);
	hAdjust = gtk_layout_get_hadjustment(GTK_LAYOUT(layout));
	gtk_range_set_adjustment(GTK_RANGE(hScrollbar), hAdjust);
	
	/*create two buttons and put them into the layout widget*/
	button = gtk_button_new_with_label("Press Me");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(OnButtonClicked), (gpointer)layout);
	gtk_layout_put(GTK_LAYOUT(layout), button, 0, 0);
		
	/*show all the widgets*/
	gtk_widget_show_all(window);
	
	/*enter the main loop*/
	gtk_main();
	
	return 0;
}
