#include <gtk/gtk.h>

gboolean OnWindowDeleteEvent(GtkWidget* widget, GdkEventAny* event, gpointer data)
{
	/*return false so that window will be destroyed*/
	return FALSE;
}

void OnWindowDestroy(GtkObject* object, gpointer data)
{
	/*exit main loop*/
	gtk_main_quit();
}

void OnButtonPosClicked(GtkButton* button, gpointer data)
{
	GtkPaned* paned;
	
	/*cast data to a GtkPaned*/
	paned = GTK_PANED(data);
	
	/*reset pane separator position by calling gtk_paned_set_position() with a negative position*/
	gtk_paned_set_position(paned, -1);
}

void OnButtonGutterClicked(GtkButton* button, gpointer data)
{
	/*track the gutter's old size with a static variable*/
	static guint16 oldSize = 20;
	guint16 currentSize;
	
	GtkPaned* paned;
	
	/*cast data to a GtkPaned*/
	paned = GTK_PANED(data);
	
	/*save the gutter's current size*/
	currentSize = paned->gutter_size;
	
	/*set the gutter's size to the old size*/
	gtk_paned_set_gutter_size(paned, oldSize);
	
	/*save the old size of the gutter*/
	oldSize = currentSize;
}

void OnButtonHandleClicked(GtkButton* button, gpointer data)
{
	/*track the handle's size with a static variable*/
	static guint16 oldSize = 15;
	guint16 currentSize;
	
	GtkPaned* paned;
	
	/*cast data to a GtkPaned*/
	paned = GTK_PANED(data);
	
	/*save the handle's current size*/
	currentSize = paned->handle_size;
	
	/*set the handle's size to the old size*/
	gtk_paned_set_handle_size(paned, oldSize);
	
	/*save the old size of the handle*/
	oldSize = currentSize;
}

int main (int argc, char** argv)
{
	GtkWidget* paned;
	GtkWidget* window;
	GtkWidget* vbox;
	GtkWidget* hbox;
	GtkWidget* leftFrame;
	GtkWidget* rightFrame;
	GtkWidget* leftLabel;
	GtkWidget* rightLabel;
	GtkWidget* buttonPos;
	GtkWidget* buttonGutter;
	GtkWidget* buttonHandle;
	
	/*initialize gtk*/
	gtk_init(&argc, &argv);
	
	/*create the top level window*/
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "Paned Example");
	gtk_window_set_default_size(GTK_WINDOW(window), 300, 300);
	gtk_signal_connect(GTK_OBJECT(window), "delete-event", GTK_SIGNAL_FUNC(OnWindowDeleteEvent), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(OnWindowDestroy), NULL);
	
	/*create vertical box and pack into window*/
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	
	/*create the horizontal box and pack into the vertical box*/
	hbox = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	
	/*create paned widget and pack into vertical box*/
	paned = gtk_hpaned_new();
	gtk_box_pack_end(GTK_BOX(vbox), paned, TRUE, TRUE, 0);
	
	/*create frames and pack into panes*/
	leftFrame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(leftFrame), GTK_SHADOW_IN);
	gtk_paned_pack1(GTK_PANED(paned), leftFrame, FALSE, TRUE);

	rightFrame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(rightFrame), GTK_SHADOW_IN);
	gtk_paned_pack2(GTK_PANED(paned), rightFrame, TRUE, FALSE);

	
	/*create labels and pack into frames*/
	leftLabel = gtk_label_new("This is the left pane.");
	gtk_container_add(GTK_CONTAINER(leftFrame), leftLabel);
	
	rightLabel = gtk_label_new("This is the right pane.");
 	gtk_container_add(GTK_CONTAINER(rightFrame), rightLabel);
 	
	/*create buttons and pack into hbox*/
	buttonPos = gtk_button_new_with_label("Position");
	gtk_signal_connect(GTK_OBJECT(buttonPos), "clicked", GTK_SIGNAL_FUNC(OnButtonPosClicked), (gpointer)paned);
	gtk_box_pack_start(GTK_BOX(hbox), buttonPos, TRUE, TRUE, 0);
		
	buttonGutter = gtk_button_new_with_label("Gutter");
	gtk_signal_connect(GTK_OBJECT(buttonGutter), "clicked", GTK_SIGNAL_FUNC(OnButtonGutterClicked), (gpointer)paned);
	gtk_box_pack_start(GTK_BOX(hbox), buttonGutter, TRUE, TRUE, 0);
		
	buttonHandle = gtk_button_new_with_label("Handle");
	gtk_signal_connect(GTK_OBJECT(buttonHandle), "clicked", GTK_SIGNAL_FUNC(OnButtonHandleClicked), (gpointer)paned);
	gtk_box_pack_start(GTK_BOX(hbox), buttonHandle, TRUE, TRUE, 0);
		
	/*show all the widgets*/
	gtk_widget_show_all(window);
	
	/*enter the main loop*/
	gtk_main();
	
	return 0;
}
