#include <gtk/gtk.h>

void button_button_release(GtkWidget *button,
     GdkEventButton *event, gpointer user_data)
{
    /* Try to receive the data from the primary selection that
       is provided by the TARGETS target. */ 
    gtk_selection_convert(button, GDK_SELECTION_PRIMARY,
        gdk_atom_intern("TARGETS", FALSE), event->time);
}

gchar *atomsToString(GdkAtom *atoms, gint format, gint length)
{
    GString *string = NULL;
    guint   counter = 0;
    gchar   *atom_name = NULL;
    
    string = g_string_new("");
    
    /* Go over every atom in the array and append it to the
       string by using a linebreak as seperator. */
    for(counter = 0; counter < length / (format/8); counter++)
    {
        /* Get the string representation of the atom. */
        atom_name = gdk_atom_name(atoms[counter]);
        if(atom_name != NULL)
        {
            /* Append the atom name to the string. */
            string = g_string_append(string, atom_name);
            string = g_string_append(string, "\n");
            g_free(atom_name);
        }
    }
    
    return string->str;
}

void button_selection_received(GtkWidget *widget,
     GtkSelectionData *data, guint time, gpointer user_data)
{
    /* Get the names that correspond to the returned atoms. */
    gchar *selection = gdk_atom_name(data->selection);
    gchar *target = gdk_atom_name(data->target);
    gchar *type = gdk_atom_name(data->type);
    
    /* Output the values of the GtkSelectionData structure. */
    g_print("selection = %s\ntarget = %s\ntype = %s\n"
            "format = %d\ndata = 0x%x\nlength = %d\n",
        selection, target, type, data->format,
        (guint)data->data, data->length);
    
    /* Free the atom names that could be retrieved. */
    if(selection) g_free(selection);
    if(target) g_free(target);
    if(type) g_free(type);
    
    /* Only process the received data if it's different from
       NULL and the length is longer than 0. It's no use to
       print nothing (length == 0).*/
    if(data->data && data->length)
    {
        g_print("%s\n", atomsToString((GdkAtom*)data->data,
            data->format ,data->length)); 
    }
    else
    {
        g_print("Couldn't get the TARGETS data.\n\n");
    }
}

int main(int argc, char *argv[])
{
    GtkWidget *window = NULL;
    GtkWidget *button = NULL;
    
    gtk_init(&argc, &argv);
    
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(window),
        "Chapter 20 : example 1");
    gtk_signal_connect(GTK_OBJECT(window), "delete_event",
        (GtkSignalFunc)gtk_exit, NULL);
    
    button = gtk_button_new_with_label(" Get TARGETS ");
    gtk_container_add(GTK_CONTAINER(window), button);
    gtk_signal_connect(GTK_OBJECT(button),
        "button_release_event",
        (GtkSignalFunc)button_button_release, NULL);
    gtk_signal_connect(GTK_OBJECT(button),
        "selection_received",
        (GtkSignalFunc)button_selection_received, NULL);
    
    gtk_widget_show_all(window);
    
    gtk_main();
    
    return 0;
}
