#include <gtk/gtk.h>
#define NO_PADDING 0
#define NO_SPACING 0

/* Callbacks */
void line_plot( GtkMenuItem *was_clicked, GtkWidget *where_to_draw );
void print_and_quit( GtkButton *was_clicked, gpointer user_data );
gboolean delete_event_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data );

gboolean dialog_delete_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data )
{
   gtk_widget_hide( widget );
   return (TRUE); /* Prevent destruction */
}

gboolean print_more_help( GtkTipsQuery *tquery, GtkWidget *widget, 
         gchar *tip_text, gchar *tip_private, GdkEventButton *event,
         gpointer user_data )
{
   GtkWidget *mhelp_dialog, *mhelp_label, *mhelp_ok;

   /* Create more help pop-up dialog */
   mhelp_dialog  = gtk_dialog_new();

   /* If no extra help, show tooltip. */
   if ( tip_private != NULL)
      mhelp_label = gtk_label_new( tip_private );
   else
      mhelp_label = gtk_label_new( tip_text );
   mhelp_ok = gtk_button_new_with_label( "OK" );

  /* Put the label in vbox, and the button in action_area */
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(mhelp_dialog)->vbox), mhelp_label,
              FALSE, FALSE, NO_PADDING );
   gtk_box_pack_start( GTK_BOX(GTK_DIALOG(mhelp_dialog)->action_area), mhelp_ok,
              FALSE, FALSE, NO_PADDING );
   gtk_widget_show( mhelp_label );
   gtk_widget_show( mhelp_ok );

   /* Destroy this widget when user is done reading it, as the
    * What's This? feature ends after they select a widget */
   gtk_signal_connect_object( GTK_OBJECT( mhelp_ok ), "clicked",
           GTK_SIGNAL_FUNC( gtk_widget_destroy ), GTK_OBJECT( mhelp_dialog ) );

   /* Show the more help dialog */
   gtk_widget_show( mhelp_dialog );
   return TRUE;
}

int main( int argc, char *argv[] )
{
  GtkWidget *top_widget;
  GtkWidget *vbox, *menu_bar, *file_item, *file_menu, *exit_item;
  GtkWidget *plot_item, *plot_area;
  GtkWidget *help_item;
  GtkWidget *help_menu, *query_item, *separator, *about_help;
  GtkWidget *about_dialog, *about_label, *about_ok;
  GtkTooltips *tooltips;
  GtkWidget *tips_query;


  /* 1. Initialize the environment */
    gtk_init( &argc, &argv );

  /* 2a. Create widgets */
    top_widget = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    vbox = gtk_vbox_new( FALSE, NO_SPACING );
    menu_bar = gtk_menu_bar_new( );

   /* Create the File item and its menu */
    file_item = gtk_menu_item_new_with_label( "File" );
    file_menu = gtk_menu_new();
     exit_item = gtk_menu_item_new_with_label( "Exit" );
 
   /* Create the Plot item */
    plot_item = gtk_menu_item_new_with_label( "Plot" );

   /* Create the Help item and its menu */
    help_item = gtk_menu_item_new_with_label( "Help" );
    help_menu = gtk_menu_new();
     query_item = gtk_menu_item_new_with_label( "What's This?");
     separator = gtk_menu_item_new();
     about_help = gtk_menu_item_new_with_label( "About my program" );

   /* Create the Work Area */
    plot_area = gtk_drawing_area_new();

   /* Create Tooltips, and the tips query widget */
    tooltips = gtk_tooltips_new();
    tips_query = gtk_tips_query_new();

   /* Create pop-up dialogs */
   about_dialog  = gtk_dialog_new();
   about_label = gtk_label_new( "This is a simple plotting program\n"
                  "It currently accepts no outside data, \n"
                  "though this feature will be available in a future release.\n"
                  "Author: You\n Version 0.0.1\n" );
   about_ok = gtk_button_new_with_label( "OK" );

  /* Put the label in vbox, and the button in action_area */
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(about_dialog)->vbox), about_label,
              FALSE, FALSE, NO_PADDING );
   gtk_box_pack_start( GTK_BOX(GTK_DIALOG(about_dialog)->action_area), about_ok,
              FALSE, FALSE, NO_PADDING );
  /* I like to show the widgets in a dialog when I create it,
   * since it's not obvious in step 5 */
   gtk_widget_show( about_label );
   gtk_widget_show( about_ok );

  /* 2b. Set Attributes */
    gtk_window_set_title( GTK_WINDOW( top_widget ), "Plotter" );
    gtk_drawing_area_size( GTK_DRAWING_AREA(plot_area), 300, 200 );

   /* Set Tooltips */
    gtk_tooltips_set_tip( tooltips, file_item, "Left Click to bring up"
                             " File Operations Menu, including Exit", 
           "Clicking (pressing and then releasing the mouse button\n"
           "while inside the File item) with your left mouse button \n "
           "will bring up a menu, which currently only contains \n"
           "one option, Exit.\nYou may select the Exit item"
           " on the resulting menu,\n"
           "with the left mouse button, as before.  The program does\n "
           "not ask if you are sure, it simply exits. ");
    gtk_tooltips_set_tip( tooltips, plot_item, "Click to plot", NULL );
    gtk_tooltips_set_tip( tooltips, help_item, "Click to bring up Help Options",
                                                NULL );
    gtk_tooltips_set_tip( tooltips, exit_item, "Click to Exit", NULL );
    gtk_tooltips_set_tip( tooltips, query_item, 
           "Get Detailed Help by Clicking on a specific widget", 
           NULL );


  /* 3. Registering Callback routines */
    gtk_signal_connect( GTK_OBJECT( about_dialog ), "delete_event",
                        GTK_SIGNAL_FUNC(dialog_delete_handler), NULL );
    gtk_signal_connect( GTK_OBJECT( top_widget ), "delete_event",
                        GTK_SIGNAL_FUNC(delete_event_handler), NULL );
    gtk_signal_connect( GTK_OBJECT( top_widget ), "destroy",
                        GTK_SIGNAL_FUNC(print_and_quit), NULL );
    gtk_signal_connect( GTK_OBJECT( exit_item ), "activate",
                        print_and_quit, NULL );
    gtk_signal_connect( GTK_OBJECT( plot_item ), "activate",
                        line_plot, plot_area );
    gtk_signal_connect_object( GTK_OBJECT( about_help ), "activate",
           GTK_SIGNAL_FUNC( gtk_widget_show ), GTK_OBJECT( about_dialog ) );
    gtk_signal_connect_object( GTK_OBJECT( about_ok ), "clicked",
           GTK_SIGNAL_FUNC( gtk_widget_hide ), GTK_OBJECT( about_dialog ) );
    gtk_signal_connect( GTK_OBJECT( tips_query ), "widget-selected",
           GTK_SIGNAL_FUNC( print_more_help ), NULL);
    gtk_signal_connect_object( GTK_OBJECT( query_item ), "activate",
           GTK_SIGNAL_FUNC( gtk_tips_query_start_query ), 
           GTK_OBJECT( tips_query ) );

  /* 4. Defining the Instance Hierarchy */
    gtk_container_add( GTK_CONTAINER( top_widget ), vbox );
    gtk_box_pack_start( GTK_BOX(vbox), menu_bar, FALSE, FALSE, NO_PADDING);
    gtk_box_pack_start( GTK_BOX(vbox), plot_area, FALSE, FALSE, NO_PADDING );
    gtk_box_pack_end( GTK_BOX(vbox), tips_query, FALSE, FALSE, NO_PADDING );

    /* Pack the Menu Bar */
    gtk_menu_bar_append( GTK_MENU_BAR( menu_bar ), file_item );
    gtk_menu_bar_append( GTK_MENU_BAR( menu_bar ), plot_item );
    gtk_menu_bar_append( GTK_MENU_BAR( menu_bar ), help_item );

    /* Attach and pack the menus */
    gtk_menu_item_set_submenu( GTK_MENU_ITEM( file_item ), file_menu );
    gtk_menu_append( GTK_MENU(file_menu), exit_item );

    gtk_menu_item_set_submenu( GTK_MENU_ITEM( help_item ), help_menu );
    gtk_menu_append( GTK_MENU(help_menu), query_item );
    gtk_menu_append( GTK_MENU(help_menu), separator );
    gtk_menu_append( GTK_MENU(help_menu), about_help );

  /* 5. Showing the Widgets */
    gtk_widget_show_all( top_widget );

  /* 6. Processing Loop */
    gtk_main();
    g_print( "Bye!\n");
    return 0;
}
