#include <gtk/gtk.h>
#define NO_PADDING 0
#define NO_SPACING 0

/* Callbacks */
void print_and_quit( GtkButton *was_clicked, gpointer user_data )
{
  /* Use glibs printf equivalent to print a message */
   g_print( "Thank you for using this program.\n");
   gtk_main_quit();
}

gboolean delete_event_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data )
{
    /* Received closure from the window manager */
    g_print("The window manager is asking to close this application\n");
    return( FALSE );  /* FALSE - do not prevent closure */
}


gboolean dialog_delete_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data )
{
   gtk_widget_hide( widget );
   return (TRUE); /* Prevent destruction */
}


gboolean print_more_help( GtkTipsQuery *tquery, GtkWidget *widget, 
         gchar *tip_text, gchar *tip_private, GdkEventButton *event,
         gpointer user_data )
{
   GtkWidget *mhelp_dialog, *mhelp_label, *mhelp_ok;

   /* Create more help pop-up dialog */
   mhelp_dialog  = gtk_dialog_new();

   /* If no extra help, show tooltip. */
   if ( tip_private != NULL)
      mhelp_label = gtk_label_new( tip_private );
   else
      mhelp_label = gtk_label_new( tip_text );
   mhelp_ok = gtk_button_new_with_label( "OK" );

  /* Put the label in vbox, and the button in action_area */
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(mhelp_dialog)->vbox), mhelp_label,
              FALSE, FALSE, NO_PADDING );
   gtk_box_pack_start( GTK_BOX(GTK_DIALOG(mhelp_dialog)->action_area), mhelp_ok,
              FALSE, FALSE, NO_PADDING );
   gtk_widget_show( mhelp_label );
   gtk_widget_show( mhelp_ok );

   /* Destroy this widget when user is done reading it, as the
    * What's This? feature ends after they select a widget */
   gtk_signal_connect_object( GTK_OBJECT( mhelp_ok ), "clicked",
           GTK_SIGNAL_FUNC( gtk_widget_destroy ), GTK_OBJECT( mhelp_dialog ) );

   /* Show the more help dialog */
   gtk_widget_show( mhelp_dialog );
   return TRUE;
}

#define X_AXIS 180
#define Y_AXIS 175
#define X_PAD 20
#define Y_PAD 10
#define NUM_PTS 10
#define X 0
#define Y 1

void line_plot( GtkMenuItem *was_clicked, GtkWidget *where_to_draw )
{
   static gint to_plot[NUM_PTS][2] = { {0, 0}, {15, 20}, {30, 80}, 
                              {45, 30}, {60, 25}, {75, 90}, {90, 120}, 
                              {105, 170}, {120, 100}, {135, 0} };
   static GdkGC *plot_gc;
   static int first_time_in = 1;
   int pts;

  if( first_time_in )
  {
    /* Initialize GC */
    plot_gc = gdk_gc_new( GTK_WIDGET(where_to_draw)->window );
  }
  /* Draw Axis */
   gdk_draw_line( GTK_WIDGET(where_to_draw)->window, plot_gc, 
                  X_PAD, Y_AXIS + Y_PAD, X_PAD, Y_PAD );
   gdk_draw_line( GTK_WIDGET(where_to_draw)->window, plot_gc,
                  X_PAD, Y_AXIS + Y_PAD, X_PAD + X_AXIS, Y_AXIS + Y_PAD );
   
   for( pts = 1; pts < NUM_PTS; pts++)
        gdk_draw_line( GTK_WIDGET(where_to_draw)->window, plot_gc,
               to_plot[pts-1][X] + X_PAD, Y_AXIS + Y_PAD - to_plot[pts-1][Y],
               to_plot[pts][X] + X_PAD, Y_AXIS + Y_PAD - to_plot[pts][Y] );

   g_print( "Thank you for plotting.\n");
}

