#include <gtk/gtk.h>
#include <stdio.h>
#define SPACING 10
#define PADDING 0

/* Callbacks */
void print_and_quit( GtkButton *was_clicked, gpointer user_data );
gboolean delete_event_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data );
gboolean dialog_delete_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data );

void ok_callback( GtkWidget *was_clicked, GtkWidget *label )
{
   FILE *msg_fd;               /* File descriptor */
   gchar *msg_file;            /* File name */
   gchar c;                    /* Character retrieved from file */
   GString *text;              /* Message text */
   GtkWidget *file_sel;        /* Parenting file selection widget */

   file_sel = gtk_widget_get_ancestor( was_clicked,
                                            GTK_TYPE_FILE_SELECTION);
   gtk_widget_hide( file_sel );
   /* Open file */
   msg_file = gtk_file_selection_get_filename( GTK_FILE_SELECTION(file_sel) );
   msg_fd = fopen( msg_file, "r" );

   /* Create an empty GString for the message text */
   text = g_string_new("");

   /* Retrieve message */
   while ((c = fgetc(msg_fd)) != EOF )
      g_string_append_c( text, c );

   /* Update the label's text */
   gtk_label_set_text( GTK_LABEL(label), text->str );

}

GtkWidget *append_menu_item( GtkMenu *menu, const gchar *text, 
                   GtkSignalFunc callback, gpointer user_data)
{
   GtkWidget *menu_item;
   if ( text )
     menu_item = gtk_menu_item_new_with_label( text );
   else
     menu_item = gtk_menu_item_new();

   gtk_menu_append( menu, menu_item );
   gtk_widget_show( menu_item );
   if (callback)
     gtk_signal_connect( GTK_OBJECT(menu_item), "activate", callback, user_data );
   return menu_item;
}

int main( int argc, char *argv[] )
{
  GtkWidget *top_widget;
  GtkWidget *vbox, *menu_bar, *file_item; 
  GtkWidget *file_menu, *change;
  GtkWidget *message, *message_fsb;
  GtkWidget *scroll;     

  /* 1. Initialize the environment */
    gtk_init( &argc, &argv );

  /* 2a. Create widgets */
    top_widget = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    vbox = gtk_vbox_new( FALSE, SPACING );
    menu_bar = gtk_menu_bar_new( );

   /* Create the File item and its menu */
    file_item = gtk_menu_item_new_with_label( "File" );
    file_menu = gtk_menu_new();
    change = append_menu_item( GTK_MENU(file_menu), "Change Message", NULL, NULL );
    append_menu_item( GTK_MENU(file_menu), NULL, NULL, NULL );
    append_menu_item( GTK_MENU(file_menu), "Exit", print_and_quit, NULL );
 
   /* Create the Label item */
    message = gtk_label_new( "Your message is:\n There is a Software Design "
                             "Meeting\n at 10:00 this morning.\n");

   /* Give the label a scrolling parent */
    scroll = gtk_scrolled_window_new( NULL, NULL );
    gtk_widget_set_usize( scroll, 200, 100 );
    gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW( scroll ), message );

   /* Create the File Selection */
    message_fsb = gtk_file_selection_new( "Select new message file");
    gtk_file_selection_hide_fileop_buttons( GTK_FILE_SELECTION(message_fsb) );
    gtk_file_selection_set_filename( GTK_FILE_SELECTION(message_fsb), "/home/messages/*" );

  /* 2b. Set Attributes */
    gtk_window_set_title( GTK_WINDOW( top_widget ), "Message" );

  /* 3. Registering Callback routines */
    gtk_signal_connect( GTK_OBJECT( top_widget ), "delete_event",
                        GTK_SIGNAL_FUNC(delete_event_handler), NULL );
    gtk_signal_connect( GTK_OBJECT( top_widget ), "destroy",
                        GTK_SIGNAL_FUNC(print_and_quit), NULL );
    gtk_signal_connect_object( GTK_OBJECT( change ), "activate",
                        GTK_SIGNAL_FUNC(gtk_widget_show), GTK_OBJECT(message_fsb) );
    gtk_signal_connect( GTK_OBJECT( message_fsb ), "delete_event",
                        GTK_SIGNAL_FUNC(dialog_delete_handler), NULL );
    gtk_signal_connect( GTK_OBJECT( GTK_FILE_SELECTION(message_fsb)->ok_button ), 
                        "clicked", ok_callback, message );
    gtk_signal_connect_object( GTK_OBJECT( 
                     GTK_FILE_SELECTION(message_fsb)->cancel_button ), 
                     "clicked", 
                     GTK_SIGNAL_FUNC(gtk_widget_hide), GTK_OBJECT(message_fsb) );

  /* 4. Defining the Instance Hierarchy */
    gtk_container_add( GTK_CONTAINER( top_widget ), vbox );
    gtk_box_pack_start( GTK_BOX(vbox), menu_bar, FALSE, FALSE, PADDING);
    gtk_box_pack_start( GTK_BOX(vbox), scroll, FALSE, FALSE, PADDING );

    /* Pack the Menu Bar */
    gtk_menu_bar_append( GTK_MENU_BAR( menu_bar ), file_item );

    /* Attach and pack the menus */
    gtk_menu_item_set_submenu( GTK_MENU_ITEM( file_item ), file_menu );

  /* 5. Showing the Widgets */
    gtk_widget_show_all( top_widget );

  /* 6. Processing Loop */
    gtk_main();
    return 0;
}
