#include <gtk/gtk.h>
#include <stdio.h>
#define SPACING 10
#define PADDING 0

/* Callbacks */
void print_and_quit( GtkButton *was_clicked, gpointer user_data );
gboolean delete_event_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data );
gboolean dialog_delete_handler( GtkWidget *widget, GdkEvent *event,
                               gpointer user_data );
void ok_callback( GtkWidget *was_clicked, GtkWidget *label );
void draw( GtkWidget *was_clicked, GtkWidget *label );
void tictactoe_grid( GtkWidget *was_clicked, GtkWidget *label );
void star( GtkWidget *was_clicked, GtkWidget *label );
void girl( GtkWidget *was_clicked, GtkWidget *label );

GtkWidget *append_menu_item( GtkMenu *menu, const gchar *text, 
                   GtkSignalFunc callback, gpointer user_data)
{
   GtkWidget *menu_item;
   if ( text )
     menu_item = gtk_menu_item_new_with_label( text );
   else
     menu_item = gtk_menu_item_new();

   gtk_menu_append( menu, menu_item );
   gtk_widget_show( menu_item );
   if (callback)
     gtk_signal_connect( GTK_OBJECT(menu_item), "activate", callback, user_data );
   return menu_item;
}
main( gint argc, gchar *argv[] )
{
  GtkWidget *top_widget;
  GtkWidget *vbox, *menu_bar, *file_item, *draw_item; 
  GtkWidget *file_menu, *draw_menu;
  GtkWidget *drawing_area;

  /* 1. Initialize the environment */
    gtk_init( &argc, &argv );

  /* 2a. Create widgets */
    top_widget = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    vbox = gtk_vbox_new( FALSE, SPACING );
    menu_bar = gtk_menu_bar_new( );

   /* Create the File item and its menu */
    file_item = gtk_menu_item_new_with_label( "File" );
    file_menu = gtk_menu_new();
    append_menu_item( GTK_MENU(file_menu), NULL, NULL, NULL );
    append_menu_item( GTK_MENU(file_menu), "Exit", print_and_quit, NULL );
 
   /* Create the work area */
    drawing_area = gtk_drawing_area_new( );
    gtk_drawing_area_size( GTK_DRAWING_AREA(drawing_area), 200, 100 );
    draw_item = gtk_menu_item_new_with_label( "Draw" );

    draw_menu = gtk_menu_new();
    append_menu_item( GTK_MENU(draw_menu), "Point", draw, drawing_area );
    append_menu_item( GTK_MENU(draw_menu), "Grid", tictactoe_grid, drawing_area );
    append_menu_item( GTK_MENU(draw_menu), "Star", star, drawing_area );
    append_menu_item( GTK_MENU(draw_menu), "Girl", girl, drawing_area );


  /* 2b. Set Attributes */
    gtk_window_set_title( GTK_WINDOW( top_widget ), "Primitives" );

  /* 3. Registering Callback routines */
    gtk_signal_connect( GTK_OBJECT( top_widget ), "delete_event",
                        GTK_SIGNAL_FUNC(delete_event_handler), NULL );
    gtk_signal_connect( GTK_OBJECT( top_widget ), "destroy",
                        GTK_SIGNAL_FUNC(print_and_quit), NULL );

  /* 4. Defining the Instance Hierarchy */
    gtk_container_add( GTK_CONTAINER( top_widget ), vbox );
    gtk_box_pack_start( GTK_BOX(vbox), menu_bar, FALSE, FALSE, PADDING);
    gtk_box_pack_start( GTK_BOX(vbox), drawing_area, FALSE, FALSE, PADDING );

    /* Pack the Menu Bar */
    gtk_menu_bar_append( GTK_MENU_BAR( menu_bar ), file_item );
    gtk_menu_bar_append( GTK_MENU_BAR( menu_bar ), draw_item );

    /* Attach and pack the menus */
    gtk_menu_item_set_submenu( GTK_MENU_ITEM( file_item ), file_menu );
    gtk_menu_item_set_submenu( GTK_MENU_ITEM( draw_item ), draw_menu );

  /* 5. Showing the Widgets */
    gtk_widget_show_all( top_widget );

  /* 6. Processing Loop */
    gtk_main();
}
